<?php

///////////////////////////////////////////////////////////////////
//
// Mark V. Shaney in PHP
//
// Copyright (c) 2009 by Yisong Yue <yisongyue@gmail.com>
// Permission to copy all or part of this work is granted, provided
// that the copies are not made or distributed for resale, and that
// the copyright notice and this notice are retained.
//
// THIS WORK IS PROVIDED ON AN "AS IS" BASIS.  THE AUTHOR PROVIDES 
// NO WARRANTY WHATSOEVER, EITHER EXPRESS OR IMPLIED, REGARDING THE 
// WORK, INCLUDING WARRANTIES WITH RESPECT TO ITS MERCHANTABILITY 
// OR FITNESS FOR ANY PARTICULAR PURPOSE.
//
//  

// Checks whether the inputted string is the end of a sentence
function checkStringSentenceEnd( $str )
{
    $len = strlen($str);
    $end = substr($str, $len-1);
    if($end == "." or $end == "!" or $end == "?")
        return 1;
    else
        return 0;
}

// Chooses a random element in the inputted array
function chooseRandom ($arr)
{
    $len = sizeof($arr);
    if($len == 0)
    {
        return -1;
    }
    return $arr[rand(0,$len-1)];
}

// Main function to use
// $words -- a string containing natural text (including punctuation)
// $numResults -- an integer indicate the number of results to generate
// returns an array of results
function runShaney( $words, $numResults )
{
    $dict = array();
    $begin = array();

    //eliminating backslashes, endlines and extra whitespaces
    $words = preg_replace("/\\\/","",$words);
    $words = preg_replace("/\n/"," ",$words);
    $words = preg_replace("/\r/"," ",$words);
    $words = preg_replace("/\f/"," ",$words);
    $words = preg_replace("/ +/"," ",$words);

    //splitting input string into array of words
    $tokens = preg_split("/ /", $words); 

    $prev1 = "";
    $prev1_end = 1;
    $prev2 = "";
    $prev2_end = 0;
    $curr = "";
    $curr_end = 0;
    foreach ($tokens as $token)
    {
        $curr = $token;
        $curr_end = checkStringSentenceEnd( $curr );
        if($prev1 != "" && $prev2 != "")
        {
            //only add transition terms that don't bridge two sentences
            if($prev1_end == 0 && $prev2_end == 0)
            {
                $key = $prev2." ".$prev1;
                if(sizeof($dict[$key]) == 0)
                {
                    //adding a term to the second order transition matrix
                    $dict[$key] = array();
                }

                // addint a transition element
                array_push($dict[$key], $curr);
            }
        }

        if($prev2_end == 1 and $prev1_end == 0) 
        {
            // adding term to the beginning of sentence array
            array_push($begin, array($prev1,$curr));
        }
        
        $prev2 = $prev1;
        $prev2_end = $prev1_end;
        $prev1 = $curr;
        $prev1_end = $curr_end;
    }

    $results = array();
    for ($i = 0; $i < $numResults; $i++)
    {
        $keyarr = chooseRandom($begin);
        $prev1 = $keyarr[1];
        $prev2 = $keyarr[0];
        $curr = chooseRandom($dict[$prev2." ".$prev1]);
        $sentence = $prev2." ".$prev1;

        //only generate sentences up to 100 words long 
        $count = 100;
        while($count > 0 && $curr != -1)
        {
            $count--;
            $sentence .= " ".$curr;
            $prev2 = $prev1;
            $prev1 = $curr;
            $curr = chooseRandom($dict[$prev2." ".$prev1]);
        }
        array_push($results, $sentence);
    }

    return $results;
}
?>
